@@ -1,4 +1,4 @@
-## Copyright 2011-2012 Luc Saffre
+## Copyright 2011-2013 Luc Saffre
 ## This file is part of the Lino project.
 ## Lino is free software; you can redistribute it and/or modify 
 ## it under the terms of the GNU General Public License as published by
@@ -11,10 +11,11 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Lino; if not, see <http://www.gnu.org/licenses/>.
 
-r"""
+"""
 A shortcut to classes and methods needed 
 when defining your database structure in a `models` module.
-The name "dd" stands for "Data Definition". 
+
+The name "dd" stands for "Data Definition".
 
 A small wrapper around Django's `Model` class 
 which adds some Lino specific features:
@@ -29,22 +30,26 @@ Tables:
 
 Extended Fields:
 
+- :class:`EnableChild <lino.utils.mti.EnableChild>`
 - :class:`NullCharField <lino.core.fields.NullCharField>`
 - :class:`IncompleteDateField <lino.core.fields.IncompleteDateField>`
 - :class:`PasswordField <lino.core.fields.PasswordField>`
 - :class:`MonthField <lino.core.fields.MonthField>`
+- :class:`PercentageField <lino.core.fields.PercentageField>`
 - :class:`QuantityField <lino.core.fields.QuantityField>`
 - :class:`PriceField<lino.core.fields.PriceField>`
 - :class:`GenericForeignKey <lino.core.fields.GenericForeignKey>`
 - :class:`GenericForeignKeyIdField <lino.core.fields.GenericForeignKeyIdField>`
 - :class:`RecurrenceField <lino.core.fields.RecurrenceField>`
 - :class:`DummyField <lino.core.fields.DummyField>`
-- :func:`ForeignKey`
+- :func:`ForeignKey <lino.core.fields.ForeignKey>`
 
 Babel fields:
 
+- :class:`BabelNamed <lino.utils.babel.BabelNamed>`
 - :class:`BabelCharField <lino.utils.babel.BabelCharField>`
 - :class:`BabelTextField <lino.utils.babel.BabelTextField>`
+- :class:`LanguageField <lino.utils.babel.LanguageField>`
 
 Virtual Fields:
 
@@ -69,21 +74,40 @@ Utilities:
 - :func:`resolve_app <lino.core.modeltools.resolve_app>` 
 - :func:`chooser <lino.utils.choosers.chooser>` 
 - :func:`add_user_group` 
-- :func:`inject_quick_add_buttons` 
-- :func:`update_field` 
-- :func:`inject_field` 
+- :func:`update_field <lino.core.inject.update_field>` 
+- :func:`inject_field <lino.core.inject.inject_field>` 
+- :func:`inject_quick_add_buttons <lino.core.inject.inject_quick_add_buttons>` 
+
+Signals:
+
+- :attr:`pre_startup <lino.core.signals.pre_startup>`
+- :attr:`pre_merge <lino.core.signals.pre_merge>`
+- :attr:`pre_ui_create <lino.core.signals.pre_ui_create>`
+- :attr:`ChangeWatcher <lino.core.signals.ChangeWatcher>`
+- :attr:`pre_ui_update <lino.core.signals.pre_ui_update>`
+- :attr:`pre_ui_delete <lino.core.signals.pre_ui_delete>`
+- :attr:`pre_analyze <lino.core.signals.pre_analyze>`
+- :attr:`receiver <django.dispatch.receiver>` : the standard Django receiver decorator
+
+Actions:
+
+- :class:`AuthorRowAction <lino.mixins.AuthorRowAction>`
+- :class:`RowAction <lino.core.actions.RowAction>`
+- :class:`AjaxAction <lino.core.actions.RowAction>`
+- :class:`ChangeStateAction <lino.core.changes.ChangeStateAction>`
+- :class:`NotifyingAction <lino.core.actions.NotifyingAction>`
+- :class:`MergeAction <lino.core.merge.MergeAction>`
 
 Miscellaneous:
 
-- :class:`UserProfiles <lino.utils.choicelists.UserProfiles>`
-- :class:`UserGroups <lino.utils.choicelists.UserGroups>`
-- :class:`UserLevels <lino.utils.choicelists.UserLevels>`
-- :class:`ChoiceList <lino.utils.choicelists.ChoiceList>`
-- :class:`Workflow <lino.utils.workflows.Workflow>`
-- :class:`ChangeStateAction <lino.core.changes.ChangeStateAction>`
-- :class:`NotifyingAction <lino.core.actions.NotifyingAction>`
+- :class:`ChoiceList <lino.core.choicelists.ChoiceList>`
+- :class:`Workflow <lino.core.workflows.Workflow>`
 
 
+- :class:`UserProfiles <lino.utils.auth.UserProfiles>`
+- :class:`UserGroups <lino.utils.auth.UserGroups>`
+- :class:`UserLevels <lino.utils.auth.UserLevels>`
+
 
 """
 
@@ -94,8 +118,11 @@ logger = logging.getLogger(__name__)
 from lino.core.tables import VirtualTable
 
 from lino.core.modeltools import resolve_model, resolve_app, resolve_field, get_field, UnresolvedModel
+from lino.core.modeltools import full_model_name
+from lino.core.modeltools import models_by_base
 
 from lino.core.model import Model
+from lino.core.merge import MergeAction
 
 #~ from lino.core.table import fields_list, inject_field
 from lino.core.dbtables import has_fk
@@ -111,18 +138,21 @@ from lino.core.dbtables import summary, summary_row
 
 from lino.core.frames import Frame
 from lino.core.dialogs import Dialog
-#~ from lino.core.frames import EmptyTable
 
 from lino.core.actions import action
 #~ from lino.core.actions import Action
 from lino.core.actions import RowAction
+AjaxAction = RowAction
+
+from lino.mixins import AuthorRowAction
+#~ from lino.core.actions import ListAction
 from lino.core.actions import GridEdit, ShowDetailAction
 from lino.core.actions import InsertRow, DeleteSelected
 from lino.core.actions import SubmitDetail, SubmitInsert
 #~ from lino.core.actions import Calendar
 
-from lino.utils.choicelists import ChoiceList, Choice
-from lino.utils.workflows import Workflow, ChangeStateAction
+from lino.core.choicelists import ChoiceList, Choice
+from lino.core.workflows import Workflow, ChangeStateAction
 #~ from lino.core.changes import ChangeStateAction
 from lino.core.actions import NotifyingAction
 
@@ -133,9 +163,11 @@ from lino.core.fields import RecurrenceField
 from lino.core.fields import GenericForeignKey
 from lino.core.fields import GenericForeignKeyIdField
 from lino.core.fields import IncompleteDateField
+from lino.core.fields import DatePickerField
 from lino.core.fields import NullCharField
 from lino.core.fields import PasswordField
 from lino.core.fields import MonthField
+from lino.core.fields import PercentageField
 #~ from lino.core.fields import LinkedForeignKey
 from lino.core.fields import QuantityField
 from lino.core.fields import HtmlBox, PriceField, RichTextField
@@ -144,12 +176,15 @@ from lino.core.fields import DisplayField, displayfield
 from lino.core.fields import VirtualField, virtualfield
 from lino.core.fields import RequestField, requestfield
 from lino.core.fields import Constant, constant
-from lino.utils.babel import BabelCharField, BabelTextField
+from lino.core.fields import ForeignKey
+
+from lino.utils.babel import BabelCharField, BabelTextField, BabelNamed, LanguageField
 #~ from lino.core.fields import MethodField
 
 from lino.utils.choosers import chooser
+from lino.utils.mti import EnableChild
 
-from lino.utils.choicelists import UserLevels, UserProfiles, UserGroups
+from lino.utils.auth import UserLevels, UserProfiles, UserGroups
 
 #~ from lino.base.utils import UserLevels, UserGroups, UserProfiles
 
@@ -161,138 +196,31 @@ from lino.core.layouts import ParamsLayout
 #~ from lino.core.layouts import DetailLayout, InsertLayout
 
 
-class Module(object):
-    pass
-    
-PENDING_INJECTS = dict()
-PREPARED_MODELS = dict()
+from lino.core.signals import pre_ui_create, pre_ui_delete, pre_ui_update, ChangeWatcher
+from lino.core.signals import pre_analyze
+from lino.core.signals import post_analyze
 
-def on_class_prepared(signal,sender=None,**kw):
-    """
-    This is Lino's general `class_prepared` handler.
-    It does two things:
-    
-    - Run pending calls to :func:`inject_field` and :func:`update_field`.
-    
-    - Apply a workaround for Django's ticket 10808.
-      In a Diamond inheritance pattern, `_meta._field_cache` contains certain fields twice.    
-      So we remove these duplicate fields from `_meta._field_cache`.
-      (A better solution would be of course to not collect them.)
-    """
-    model = sender
-    #~ return
-    #~ if model is None:
-        #~ return 
-    k = model._meta.app_label + '.' + model.__name__
-    PREPARED_MODELS[k] = model
-    #~ logger.info("20120627 on_class_prepared %r = %r",k,model)
-    todos = PENDING_INJECTS.pop(k,None)
-    if todos is not None:
-        for f in todos:
-            f(model)
-        #~ for k,v in injects.items():
-            #~ model.add_to_class(k,v)
+from lino.core.signals import auto_create
+from lino.core.signals import pre_merge
+from lino.core.signals import pre_add_child
+from lino.core.signals import pre_remove_child
+
+from django.db.models.signals import class_prepared
+from django.db.models.fields import NOT_PROVIDED
+
+from django.dispatch import receiver
+#~ from lino.core import signals
 
 
-    if hasattr(model._meta,'_field_cache'):
-        cache = []
-        field_names = set()
-        for f,m in model._meta._field_cache:
-            if f.attname not in field_names:
-                field_names.add(f.attname)
-                cache.append( (f,m) )
-        model._meta._field_cache = tuple(cache)
-        model._meta._field_name_cache = [x for x, _ in cache]
-    #~ else:
-        #~ logger.info("20120905 Could not fix Django issue #10808 for %s",model)
+#~ class Module(object):
+    #~ pass
 
-
-models.signals.class_prepared.connect(on_class_prepared)
-
-def do_when_prepared(model_spec,todo):
-    if model_spec is None:
-        return # e.g. inject_field during autodoc when user_model is None
-        
-    if isinstance(model_spec,basestring):
-        app_label, model_name = model_spec.split(".")
-        if not settings.LINO.is_installed(app_label):
-            return
-        k = model_spec
-        model = PREPARED_MODELS.get(k,None)
-        if model is None: 
-            injects = PENDING_INJECTS.setdefault(k,[])
-            injects.append(todo)
-            #~ d[name] = field
-            #~ logger.info("20120627 Defer inject_field(%r,%r,%r)", model_spec,name,field)
-            return
-    else:
-        model = model_spec
-        #~ k = model_spec._meta.app_label + '.' + model_spec.__name__
-    todo(model)
-
-
-def inject_field(model_spec,name,field,doc=None):
-    """
-    Adds the given field to the given model.
-    See also :doc:`/tickets/49`.
-    
-    Since `inject_field` is usually called at the global level 
-    of `models modules`, it cannot know whether the given `model_spec` 
-    has already been imported (and its class prepared) or not. 
-    That's why it uses Django's `class_prepared` signal to maintain 
-    its own list of models.
-    """
-    if doc:
-        field.__doc__ = doc
-    def todo(model):
-        model.add_to_class(name,field)
-    return do_when_prepared(model_spec,todo)    
-    
-    
-
-def update_field(model_spec,name,**kw):
-    """
-    Update some attribute of the specified existing field.
-    For example 
-    :class:`PersonMixin <lino.modlib.contacts.models.PersonMixin>` 
-    defines a field `first_name` which may not be blank.
-    If you inherit from 
-    :class:`PersonMixin <lino.modlib.contacts.models.PersonMixin>`
-    but want `first_name` to be optional::
-    
-      class MyPerson(contacts.PersonMixin):
-        ...
-      dd.update_field(MyPerson,'first_name',blank=True)
-    
-    """
-    def todo(model):
-        try:
-            fld = model._meta.get_field(name)
-            #~ fld = model._meta.get_field_by_name(name)[0]
-        except FieldDoesNotExist:
-            logger.warning("Cannot update unresolved field %s.%s", model,name)
-            return
-        if fld.model != model:
-            logger.warning('20120715 update_field(%s.%s) : %s',model,fld,fld.model)
-        for k,v in kw.items():
-            setattr(fld,k,v)
-    return do_when_prepared(model_spec,todo)    
-        
-
-def inject_quick_add_buttons(model,name,target):
-    """
-    Injects a virtual display field `name` into the specified `model`.
-    This field will show up to three buttons
-    `[New]` `[Show last]` `[Show all]`. 
-    `target` is the table that will run these actions.
-    It must be a slave of `model`.
-    """
-    def fn(self,rr):
-        return rr.renderer.quick_add_buttons(
-          rr.spawn(target,master_instance=self))
-    inject_field(model,name,
-        VirtualField(DisplayField(
-            target.model._meta.verbose_name_plural),fn))
+from lino.core.inject import inject_field
+from lino.core.inject import update_field
+from lino.core.inject import inject_quick_add_buttons
+from lino.core.inject import do_when_prepared, when_prepared
+ 
+  
     
 def add_user_group(name,label):
     """
@@ -309,17 +237,23 @@ def add_user_group(name,label):
         if g.text != label:
             g.text += " & " + unicode(label)
     
-def ForeignKey(othermodel,*args,**kw):
-    """
-    This is exactly the same as 
-    `django.db.models.ForeignKey <https://docs.djangoproject.com/en/dev/ref/models/fields/#foreignkey>`, 
-    except for a subtle difference: it supports `othermodel` being `None`
-    and returns a :class:`DummyField <lino.core.fields.DummyField>` 
-    in that case.
-    This difference is useful when designing reusable models.
-    """
-    if othermodel is None: 
-        return DummyField(othermodel,*args,**kw)
-    return models.ForeignKey(othermodel,*args,**kw)
+def required(**kw):
+    #~ if not kw.has_key('auth'):
+        #~ kw.update(auth=True)
+    kw.setdefault('auth',True)
+    return kw
     
-    
\ No newline at end of file
+    
+class PseudoRequest:
+    def __init__(self,username):
+        self.username = username
+        self._user = None
+        
+    def get_user(self):
+        if self._user is None:
+            if settings.LINO.user_model is not None:
+                self._user = settings.LINO.user_model.objects.get(username=self.username)
+        return self._user
+    user = property(get_user)
+    
+
