@@ -13,13 +13,15 @@
 ## along with Lino; if not, see <http://www.gnu.org/licenses/>.
 
 """
-This module deserves more documentation.
-
-It defines tables like 
+This module defines the tables 
 - :class:`Partner` (and their specializations :class:`Person` and :class:`Company`)
+- :class:`Role` and :class:`RoleType`
 
 """
 
+from __future__ import unicode_literals
+
+
 import logging
 logger = logging.getLogger(__name__)
 
@@ -62,6 +64,7 @@ from lino.modlib.countries.models import CountryRegionCity
 from lino.utils import mti
 
 
+from lino.modlib.contacts import MODULE_LABEL
 
 
 class CompanyType(babel.BabelNamed):
@@ -150,12 +153,12 @@ class Partner(mti.MultiTableBase,CountryRegionCity):
     remarks = models.TextField(_("Remarks"),blank=True) # ,null=True)
     
     is_person = mti.EnableChild(
-        settings.LINO.person_model,
+        "contacts.Person",
         verbose_name=_("is Person"),
         help_text=_("Whether this Partner is a Person."))
         
     is_company = mti.EnableChild(
-        settings.LINO.company_model,
+        "contacts.Company",
         verbose_name=_("is Company"),
         help_text=_("Whether this Partner is a Company."))
         
@@ -405,7 +408,7 @@ class Persons(Partners):
     List of all Persons.
     """
     #~ required = dict(user_level='user')
-    model = settings.LINO.person_model
+    model = "contacts.Person"
     order_by = ["last_name","first_name","id"]
     column_names = "name_column:20 address_column email phone:10 gsm:10 id language:10 *"
     detail_layout = PersonDetail()
@@ -453,18 +456,20 @@ class Company(Partner):
     #~ def site_setup(cls,lino):
         #~ raise Exception('20110810')
 
-
 class CompanyDetail(PartnerDetail):
   
-    bottom_box = "remarks contacts.RolesByCompany"
+    bottom_box = """
+    type vat_id:12
+    remarks contacts.RolesByCompany
+    """
 
-    name_box = """prefix name type:20"""
-    info_box = "id:5 language:10 vat_id:12"
+    #~ name_box = """prefix name type:20"""
+    #~ info_box = "id:5 language:10 vat_id:12"
 
 
 
 class Companies(Partners):
-    model = settings.LINO.company_model
+    model = "contacts.Company"
     order_by = ["name"]
     detail_layout = CompanyDetail()
     insert_layout = dd.FormLayout("""
@@ -528,8 +533,8 @@ class Role(dd.Model):
     type = models.ForeignKey('contacts.RoleType',
       blank=True,null=True,
       verbose_name=_("Contact Role"))
-    person = models.ForeignKey(settings.LINO.person_model,related_name='rolesbyperson')
-    company = models.ForeignKey(settings.LINO.company_model,related_name='rolesbycompany')
+    person = models.ForeignKey("contacts.Person",related_name='rolesbyperson')
+    company = models.ForeignKey("contacts.Company",related_name='rolesbycompany')
     #~ type = models.ForeignKey('contacts.ContactType',blank=True,null=True,
       #~ verbose_name=_("contact type"))
 
@@ -587,10 +592,12 @@ class Roles(dd.Table):
     
 class RolesByCompany(Roles):
     required = dd.required()
+    auto_fit_column_widths = True
     #~ required_user_level = None
     label = _("Contact persons")
     master_key = 'company'
     column_names = 'person type *'
+    hidden_columns = 'id'
 
 class RolesByPerson(Roles):
     required = dd.required()
@@ -598,6 +605,8 @@ class RolesByPerson(Roles):
     label = _("Contact for")
     master_key = 'person'
     column_names = 'company type *'
+    auto_fit_column_widths = True
+    hidden_columns = 'id'
     
     
     
@@ -612,11 +621,10 @@ class PartnerDocument(dd.Model):
     class Meta:
         abstract = True
         
-    company = models.ForeignKey(settings.LINO.company_model,
+    company = models.ForeignKey("contacts.Company",
         blank=True,null=True)
         
-    person = models.ForeignKey(settings.LINO.person_model,
-        blank=True,null=True)
+    person = models.ForeignKey("contacts.Person",blank=True,null=True)
         
     def get_partner(self):
         if self.company is not None:
@@ -668,11 +676,9 @@ class PartnerDocument(dd.Model):
     def update_owned_instance(self,other):
         #~ print '20120627 PartnerDocument.update_owned_instance'
         if isinstance(other,mixins.ProjectRelated):
-            # the following hack doesn't work when loading data by dumpy
-            # because LINO.person_model are still strings at that moment
-            if isinstance(self.person,settings.LINO.person_model):
+            if isinstance(self.person,Person):
                 other.project = self.person
-            elif isinstance(self.company,settings.LINO.person_model):
+            elif isinstance(self.company,Person):
                 other.project = self.company
         other.person = self.person
         other.company = self.company
@@ -688,7 +694,7 @@ class OldCompanyContact(dd.Model):
     class Meta:
         abstract = True
         
-    company = models.ForeignKey(settings.LINO.company_model,
+    company = models.ForeignKey("contacts.Company",
         related_name="%(app_label)s_%(class)s_set_by_company",
         verbose_name=_("Company"),
         blank=True,null=True)
@@ -731,7 +737,7 @@ class ContactRelated(dd.Model):
     class Meta:
         abstract = True
         
-    company = models.ForeignKey(settings.LINO.company_model,
+    company = models.ForeignKey("contacts.Company",
         related_name="%(app_label)s_%(class)s_set_by_company",
         verbose_name=_("Company"),
         blank=True,null=True)
@@ -770,7 +776,7 @@ class ContactRelated(dd.Model):
     @classmethod
     def contact_person_choices_queryset(cls,company):
     #~ def contact_choices_queryset(cls,company):
-        return settings.LINO.person_model.objects.filter(rolesbyperson__company=company).distinct()
+        return Person.objects.filter(rolesbyperson__company=company).distinct()
 
     def full_clean(self,*args,**kw):
         if not settings.LINO.loading_from_dump:
@@ -806,23 +812,25 @@ class ContactRelated(dd.Model):
 
 
 
-from lino.models import SiteConfig
+if settings.LINO.is_installed('contacts'):
+  
+    from lino.models import SiteConfig
 
-dd.inject_field(SiteConfig,
-    'next_partner_id',
-    models.IntegerField(default=100, # first 100 for users from demo fixtures.
-        verbose_name=_("The next automatic id for Person or Company")
-    ),"""The next automatic id for Person or Company. 
-    Deserves more documentation.
-    """)
-    
-dd.inject_field(SiteConfig,
-    'site_company',
-    models.ForeignKey(settings.LINO.company_model,
-        blank=True,null=True,
-        verbose_name=_("The company that runs this site"),
-        related_name='site_company_sites',
-        help_text=_("The Company to be used as sender in documents.")))
+    dd.inject_field(SiteConfig,
+        'next_partner_id',
+        models.IntegerField(default=100, # first 100 for users from demo fixtures.
+            verbose_name=_("The next automatic id for Person or Company")
+        ),"""The next automatic id for Person or Company. 
+        Deserves more documentation.
+        """)
+        
+    dd.inject_field(SiteConfig,
+        'site_company',
+        models.ForeignKey("contacts.Company",
+            blank=True,null=True,
+            verbose_name=_("The company that runs this site"),
+            related_name='site_company_sites',
+            help_text=_("The Company to be used as sender in documents.")))
     
 
 #~ dd.inject_field(Partner,
@@ -834,14 +842,14 @@ dd.inject_field(SiteConfig,
 #~ dd.inject_field(Partner,
     #~ 'is_company',
     #~ mti.EnableChild(
-        #~ settings.LINO.company_model,
+        #~ "contacts.Company",
         #~ verbose_name=_("is Company"),
         #~ help_text=_("Whether this Partner is a Company.")))
 
 
-MODULE_LABEL = _("Contacts")
 
 def site_setup(site):
+
     site.modules.countries.Cities.set_detail_layout("""
     name country 
     type parent zip_code id 
@@ -849,6 +857,19 @@ def site_setup(site):
     contacts.PartnersByCity
     """)
     
+@dd.receiver(dd.pre_analyze)
+def company_model_alias(sender,**kw):
+    """
+    prepare ticket #72 which will rename Company to Organisation
+    """
+    sender.modules.contacts.Organisation = sender.modules.contacts.Company  
+
+@dd.receiver(dd.post_analyze)
+def company_tables_alias(sender,**kw):
+    """
+    prepare ticket #72 which will rename Company to Organisation
+    """
+    sender.modules.contacts.Organisations = sender.modules.contacts.Companies
 
 
 def setup_main_menu(site,ui,profile,m):
@@ -875,9 +896,9 @@ def setup_config_menu(site,ui,profile,m):
     config_contacts = m.add_menu("contacts",MODULE_LABEL)
     config_contacts.add_action(CompanyTypes)
     config_contacts.add_action(RoleTypes)
-    config_contacts.add_action(site.modules.countries.Countries)
-    config_contacts.add_action(site.modules.countries.Cities)
-    config_contacts.add_action(site.modules.countries.Languages)
+    #~ config_contacts.add_action(site.modules.countries.Countries)
+    #~ config_contacts.add_action(site.modules.countries.Cities)
+    #~ config_contacts.add_action(site.modules.countries.Languages)
             
     #~ m  = m.add_menu("contacts",_("~Contacts"))
     #~ m.add_action('contacts.RoleTypes')
@@ -885,7 +906,7 @@ def setup_config_menu(site,ui,profile,m):
 def setup_explorer_menu(site,ui,profile,m):
     m = m.add_menu("contacts",MODULE_LABEL)
     m.add_action(site.modules.contacts.Roles)
-    m.add_action(site.modules.countries.Cities)
+    #~ m.add_action(site.modules.countries.Cities)
   
 #~ def setup_quicklinks(site,ui,user,m):
     #~ m.add_action(Person.detail_action)
